! Stomate: phenology, allocation, etc.
!
! authors: A. Botta, P. Friedlingstein, C. Morphopoulos, N. Viovy, et al.
!
! bits and pieces put together by G. Krinner
!
! version 0.0: August 1998
!
! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/stomate_lpj.f90,v 1.26 2010/08/05 16:02:37 ssipsl Exp $
! IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
MODULE stomate_lpj

  ! modules used:

  USE ioipsl
  USE grid
  USE stomate_constants
  USE lpj_constraints
  USE lpj_pftinout
  USE lpj_kill
  USE lpj_crown
  USE lpj_fire
  USE lpj_gap
  USE lpj_light
  USE lpj_establish
  USE lpj_cover
  USE stomate_prescribe
  USE stomate_phenology
  USE stomate_alloc
  USE stomate_npp
  USE stomate_turnover
  USE stomate_litter
  USE stomate_soilcarbon
  USE stomate_vmax
  USE stomate_assimtemp
  USE constantes_veg
  USE stomate_lcchange
  !  USE Write_Field_p

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC StomateLpj,StomateLpj_clear

  ! first call
  LOGICAL, SAVE                                           :: firstcall = .TRUE.

CONTAINS

  SUBROUTINE StomateLpj_clear

    CALL prescribe_clear
    CALL phenology_clear
    CALL npp_calc_clear
    CALL turn_clear
    CALL soilcarbon_clear
    CALL constraints_clear
    CALL establish_clear
    CALL fire_clear
    CALL gap_clear
    CALL light_clear
    CALL pftinout_clear
    CALL alloc_clear
  END SUBROUTINE StomateLpj_clear

  SUBROUTINE StomateLpj (npts, dt_days, EndOfYear, EndOfMonth, &
       neighbours, resolution, &
       clay, herbivores, &
       tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
       litterhum_daily, soilhum_daily, &
       maxmoiavail_lastyear, minmoiavail_lastyear, &
       gdd0_lastyear, precip_lastyear, &
       moiavail_month, moiavail_week, tlong_ref, t2m_month, t2m_week, &
       tsoil_month, soilhum_month, &
       gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
       turnover_longterm, gpp_daily, time_lowgpp, &
       time_hum_min, maxfpc_lastyear, resp_maint_part, &
       PFTpresent, age, fireindex, firelitter, &
       leaf_age, leaf_frac, biomass, ind, adapted, regenerate, &
       senescence, when_growthinit, &
       litterpart, litter, dead_leaves, carbon, black_carbon, lignin_struc, &
       veget_max, veget, npp_longterm, lm_lastyearmax, veget_lastlight, &
       everywhere, need_adjacent, RIP_time, &
       lai, rprof,npp_daily, turnover_daily, turnover_time,&
       control_moist, control_temp, soilcarbon_input, &
       co2_to_bm, co2_fire, resp_hetero, resp_maint, resp_growth, &
       height, deadleaf_cover, vcmax, vjmax, &
       t_photo_min, t_photo_opt, t_photo_max,bm_to_litter, &
       prod10,prod100,flux10, flux100, veget_max_new, &
       convflux,cflux_prod10,cflux_prod100, harvest_above, lcchange)

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER(i_std), INTENT(in)                                           :: npts
    ! time step of Stomate in days
    REAL(r_std), INTENT(in)                                        :: dt_days
    ! indices of the 8 neighbours of each grid point (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    INTEGER(i_std), DIMENSION(npts,8), INTENT(in)                 :: neighbours
    ! resolution at each grid point in m (1=E-W, 2=N-S)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                     :: resolution
    ! clay fraction
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: clay
    ! time constant of probability of a leaf to be eaten by a herbivore (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: herbivores
    ! daily surface temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: tsurf_daily
    ! daily soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                  :: tsoil_daily
    ! daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: t2m_daily
    ! daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: t2m_min_daily
    ! daily litter humidity
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: litterhum_daily
    ! daily soil humidity
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                  :: soilhum_daily
    ! last year's maximum moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: maxmoiavail_lastyear
    ! last year's minimum moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: minmoiavail_lastyear
    ! last year's GDD0
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: gdd0_lastyear
    ! lastyear's precipitation (mm/year)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: precip_lastyear
    ! "monthly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: moiavail_month
    ! "weekly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: moiavail_week
    ! "long term" 2 meter reference temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: tlong_ref
    ! "monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: t2m_month
    ! "weekly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                       :: t2m_week
    ! "monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                  :: tsoil_month
    ! "monthly" soil humidity
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                  :: soilhum_month
    ! growing degree days, threshold -5 deg C (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                   :: gdd_m5_dormance
    ! growing degree days, since midwinter (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                   :: gdd_midwinter
    ! number of chilling days, since leaves were lost (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: ncd_dormance
    ! number of growing days, threshold -5 deg C (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: ngd_minus5
    ! "long term" turnover rate (gC/(m**2 of ground)/year)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)           :: turnover_longterm
    ! daily gross primary productivity (gC/(m**2 of ground)/day)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: gpp_daily
    ! duration of dormance (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: time_lowgpp
    ! time elapsed since strongest moisture availability (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: time_hum_min
    ! last year's maximum fpc for each natural PFT, on ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                  :: maxfpc_lastyear
    ! maintenance respiration of different plant parts (gC/day/m**2 of ground)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)             :: resp_maint_part

    ! 0.2 modified fields

    ! PFT exists
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                  :: PFTpresent
    ! age (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: age
    ! Probability of fire
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)          :: fireindex
    ! Longer term litter above the ground, gC/m**2 of ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)          :: firelitter
    ! leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)     :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)     :: leaf_frac
    ! biomass (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)        :: biomass
    ! density of individuals (1/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: ind
    ! Winter too cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: adapted
    ! Winter sufficiently cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: regenerate
    ! is the plant senescent? (only for deciduous trees - carbohydrate reserve)
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                  :: senescence
    ! how many days ago was the beginning of the growing season
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: when_growthinit
    ! fraction of litter above the ground belonging to different PFTs
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)         :: litterpart
    ! metabolic and structural litter, above and below ground (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs), INTENT(inout)  :: litter
    ! dead leaves on ground, per PFT, metabolic and structural, 
    !   in gC/(m**2 of ground)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(inout)         :: dead_leaves
    ! carbon pool: active, slow, or passive,(gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(inout)    :: carbon
    ! black carbon on the ground (gC/(m**2 of total ground))
    REAL(r_std), DIMENSION(npts), INTENT(inout)                    :: black_carbon
    ! ratio Lignine/Carbon in structural litter, above and below ground,
    ! (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)    :: lignin_struc
    ! "maximal" coverage fraction of a PFT (LAI -> infinity) on ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: veget_max
    ! fractional coverage on ground, taking into account LAI (=grid-scale fpc)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: veget
    ! "long term" net primary productivity (gC/(m**2 of ground)/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: npp_longterm
    ! last year's maximum leaf mass, for each PFT (gC/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: lm_lastyearmax
    ! vegetation fractions (on ground) after last light competition
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: veget_lastlight
    ! is the PFT everywhere in the grid box or very localized (after its introduction)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: everywhere
    ! in order for this PFT to be introduced, does it have to be present in an
    !   adjacent grid box?
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)                  :: need_adjacent
    ! How much time ago was the PFT eliminated for the last time (y)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)               :: RIP_time
    ! Turnover_time of leaves for grasses (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)          :: turnover_time

    ! 0.3 output

    ! leaf area index OF AN INDIVIDUAL PLANT
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: lai
    ! root depth. This will, one day, be a prognostic variable. It will be calculated by
    ! STOMATE (save in restart file & give to hydrology module!), probably somewhere
    ! in the allocation routine. For the moment, it is prescribed.
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: rprof
    ! net primary productivity (gC/day/(m**2 of ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: npp_daily
    ! Turnover rates (gC/(m**2 of ground)/day)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(out)          :: turnover_daily
    ! moisture control of heterotrophic respiration
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)                :: control_moist
    ! temperature control of heterotrophic respiration, above and below
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(inout)                :: control_temp
    ! quantity of carbon going into carbon pools from litter decomposition
    !   (gC/(m**2 of ground)/day)
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(inout)      :: soilcarbon_input
    ! co2 taken up (gC/(m**2 of total ground)/day)
    !NV devient 2D
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                      :: co2_to_bm
    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2 of total ground/day)
    !NV devient 2D
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                      :: co2_fire
    ! heterotrophic respiration (gC/day/m**2 of total ground)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)            :: resp_hetero
    ! maintenance respiration (gC/day/(m**2 of total ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: resp_maint
    ! growth respiration (gC/day/(m**2 of total ground))
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: resp_growth
    ! height of vegetation (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: height
    ! fraction of soil covered by dead leaves
    REAL(r_std), DIMENSION(npts), INTENT(inout)                      :: deadleaf_cover
    ! Maximum rate of carboxylation
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: vcmax
    ! Maximum rate of RUbp regeneration
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: vjmax
    ! Minimum temperature for photosynthesis (deg C)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: t_photo_min
    ! Optimum temperature for photosynthesis (deg C)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: t_photo_opt
    ! Maximum temperature for photosynthesis (deg C)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: t_photo_max
    ! conversion of biomass to litter (gC/(m**2 of ground)) / day
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(out)          :: bm_to_litter
    !
    ! new "maximal" coverage fraction of a PFT (LAI -> infinity)
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)                 :: veget_max_new 

    ! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
    ! (10 or 100 + 1 : input from year of land cover change)
    REAL(r_std),DIMENSION(npts,0:10), INTENT(inout)                        :: prod10
    REAL(r_std),DIMENSION(npts,0:100), INTENT(inout)                       :: prod100
    ! annual release from the 10/100 year-turnover pool compartments
    REAL(r_std),DIMENSION(npts,10), INTENT(inout)                       :: flux10
    REAL(r_std),DIMENSION(npts,100), INTENT(inout)                      :: flux100
    ! release during first year following land cover change
    REAL(r_std),DIMENSION(npts), INTENT(inout)                          :: convflux
    ! total annual release from the 10/100 year-turnover pool
    REAL(r_std),DIMENSION(npts), INTENT(inout)                          :: cflux_prod10, cflux_prod100
    ! harvest above ground biomass for agriculture
    REAL(r_std), DIMENSION(npts), INTENT(inout)                       :: harvest_above

    ! land cover change flag
    LOGICAL, INTENT(in)                                                :: lcchange

    ! Land cover change variables + EndOfYear
    ! Do update of yearly variables? This variable must be .TRUE. once a year
    LOGICAL, INTENT(in)                                                :: EndOfYear
    ! Do update of monthly variables ? This variable must be .TRUE. once a month
    LOGICAL, INTENT(in)                                                :: EndOfMonth


    ! 0.4 local

    ! total conversion of biomass to litter (gC/(m**2)) / day
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_bm_to_litter
    ! total living biomass (gC/(m**2)) 
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_live_biomass
    ! biomass increase, i.e. NPP per plant part
    REAL(r_std), DIMENSION(npts,nvm,nparts)                            :: bm_alloc
    ! total turnover rate (gC/(m**2)) / day
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_turnover
    ! total soil and litter carbon (gC/(m**2))
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_litter_soil_carb
    ! total litter carbon (gC/(m**2))
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_litter_carb
    ! total soil carbon (gC/(m**2))
    REAL(r_std), DIMENSION(npts,nvm)                                   :: tot_soil_carb
    ! crown area of individuals (m**2)
    REAL(r_std), DIMENSION(npts,nvm)                               :: cn_ind
    ! fraction that goes into plant part
    REAL(r_std), DIMENSION(npts,nvm,nparts)                        :: f_alloc
    ! space availability for trees
    REAL(r_std), DIMENSION(npts)                                   :: avail_tree
    ! space availability for grasses
    REAL(r_std), DIMENSION(npts)                                   :: avail_grass

    INTEGER                                                       :: j

    ! total products remaining in the pool after the annual release 
    REAL(r_std),DIMENSION(npts)                                   :: prod10_total, prod100_total 
    ! total flux from conflux and the 10/100 year-turnover pool 
    REAL(r_std),DIMENSION(npts)                                       :: cflux_prod_total

    ! "maximal" coverage fraction of a PFT (LAI -> infinity) on ground
    REAL(r_std),DIMENSION(npts,nvm)                                :: veget_max_old

    REAL(r_std), DIMENSION(npts)                                   :: vartmp

    REAL(r_std), DIMENSION(npts,nvm)                          :: histvar

    ! =========================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering stomate_lpj'

    !
    ! 1 Initializations
    !

    !
    ! 1.1 set outputs to zero
    !
    co2_to_bm(:,:) = zero
    co2_fire(:,:) = zero
    npp_daily(:,:) = zero
    turnover_daily(:,:,:) = zero
    resp_maint(:,:) = zero
    resp_growth(:,:) = zero
    harvest_above(:) = zero

    !
    ! 1.2  initialize some variables
    !

    bm_to_litter(:,:,:) = zero
    cn_ind(:,:) = zero
    veget_max_old(:,:) = veget_max(:,:)

    !
    ! 1.3 Prescribe some vegetation characteristics if the vegetation is not dynamic
    !     IF the DGVM is not activated, the density of individuals and their crown
    !     areas don't matter, but they should be defined for the case we switch on
    !     the DGVM afterwards.
    !     At first call, if the DGVM is not activated, impose a minimum biomass for 
    !       prescribed PFTs and declare them present.
    !

    CALL prescribe (npts, &
         veget_max, PFTpresent, everywhere, when_growthinit, &
         biomass, leaf_frac, ind, cn_ind)

    !
    ! 2 climatic constraints for PFT presence and regenerativeness
    !   call this even when DGVM is not activated so that "adapted" and "regenerate"
    !   are kept up to date for the moment when the DGVM is activated.
    !

    CALL constraints (npts, dt_days, &
         t2m_month, t2m_min_daily, when_growthinit, &
         adapted, regenerate)

    !
    ! 3 PFTs in and out, based on climate criteria
    !

    IF ( control%ok_dgvm ) THEN

       !
       ! 3.1 do introduction/elimination
       !

       CALL pftinout (npts, dt_days, adapted, regenerate, &
            neighbours, veget, veget_max, &
            biomass, ind, age, leaf_frac, npp_longterm, lm_lastyearmax, senescence, &
            PFTpresent, everywhere, when_growthinit, need_adjacent, RIP_time, &
            co2_to_bm, &
            avail_tree, avail_grass)

       !
       ! 3.2 reset attributes for eliminated PFTs.
       !     This also kills PFTs that had 0 leafmass during the last year. The message
       !     "... after pftinout" is misleading in this case.
       !

       CALL kill (npts, 'pftinout  ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, &
            when_growthinit, everywhere, veget, veget_max, bm_to_litter)

       !
       ! 3.3 calculate new crown area and maximum vegetation cover
       !

       CALL crown (npts, PFTpresent, &
            ind, biomass, &
            veget_max, cn_ind, height)

    ENDIF

    !
    ! 4 phenology
    !
    CALL histwrite (hist_id_stomate, 'WHEN_GROWTHINIT', itime, when_growthinit, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'TIME_LOWGPP', itime, time_lowgpp, npts*nvm, horipft_index)

    WHERE(PFTpresent)
       histvar=un
    ELSEWHERE
       histvar=zero
    ENDWHERE
    CALL histwrite (hist_id_stomate, 'PFTPRESENT', itime, histvar, npts*nvm, horipft_index)

    WHERE(gdd_midwinter.EQ.undef)
       histvar=val_exp
    ELSEWHERE
       histvar=gdd_midwinter
    ENDWHERE
    CALL histwrite (hist_id_stomate, 'GDD_MIDWINTER', itime, histvar, npts*nvm, horipft_index)

    WHERE(ncd_dormance.EQ.undef)
       histvar=val_exp
    ELSEWHERE
       histvar=ncd_dormance
    ENDWHERE
    CALL histwrite (hist_id_stomate, 'NCD_DORMANCE', itime, histvar, npts*nvm, horipft_index)

    CALL phenology (npts, dt_days, PFTpresent, &
         veget_max, &
         tlong_ref, t2m_month, t2m_week, gpp_daily, &
         maxmoiavail_lastyear, minmoiavail_lastyear, &
         moiavail_month, moiavail_week, &
         gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
         senescence, time_lowgpp, time_hum_min, &
         biomass, leaf_frac, leaf_age, &
         when_growthinit, co2_to_bm, lai)

    !
    ! 5 allocation
    !

    CALL alloc (npts, dt_days, &
         lai, veget_max, senescence, when_growthinit, &
         moiavail_week, tsoil_month, soilhum_month, &
         biomass, age, leaf_age, leaf_frac, rprof, f_alloc)

    !
    ! 6 maintenance and growth respiration. NPP
    !

    CALL npp_calc (npts, dt_days, &
         PFTpresent, &
         tlong_ref, t2m_daily, tsoil_daily, lai, rprof, &
         gpp_daily, f_alloc, bm_alloc, resp_maint_part,&
         biomass, leaf_age, leaf_frac, age, &
         resp_maint, resp_growth, npp_daily)

    IF ( control%ok_dgvm ) THEN

       ! new provisional crown area and maximum vegetation cover after growth

       CALL crown (npts, PFTpresent, &
            ind, biomass, &
            veget_max, cn_ind, height)

    ENDIF

    !
    ! 7 fire.
    !

    CALL fire (npts, dt_days, litterpart, &
         litterhum_daily, t2m_daily, lignin_struc, &
         fireindex, firelitter, biomass, ind, &
         litter, dead_leaves, bm_to_litter, black_carbon, &
         co2_fire)

    IF ( control%ok_dgvm ) THEN

       ! reset attributes for eliminated PFTs

       CALL kill (npts, 'fire      ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, &
            when_growthinit, everywhere, veget, veget_max, bm_to_litter)

    ENDIF

    !
    ! 8 tree mortality. Does not depend on age, therefore does not change crown area.
    !

    CALL gap (npts, dt_days, &
         npp_longterm, turnover_longterm, lm_lastyearmax, &
         PFTpresent, biomass, ind, bm_to_litter)

    IF ( control%ok_dgvm ) THEN

       ! reset attributes for eliminated PFTs

       CALL kill (npts, 'gap       ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, &
            when_growthinit, everywhere, veget, veget_max, bm_to_litter)

    ENDIF

    !
    ! 9 calculate vcmax, vjmax and photosynthesis temperatures
    !

    CALL vmax (npts, dt_days, &
         leaf_age, leaf_frac, &
         vcmax, vjmax)

    CALL assim_temp (npts, tlong_ref, t2m_month, &
         t_photo_min, t_photo_opt, t_photo_max)

    !
    ! 10 leaf senescence and other turnover processes. New lai
    !

    CALL turn (npts, dt_days, PFTpresent, &
         herbivores, &
         maxmoiavail_lastyear, minmoiavail_lastyear, &
         moiavail_week,  moiavail_month,tlong_ref, t2m_month, t2m_week, veget_max, &
         leaf_age, leaf_frac, age, lai, biomass, &
         turnover_daily, senescence,turnover_time)

    !
    ! 11 light competition
    !

    IF ( control%ok_dgvm ) THEN

       !
       ! 11.1 do light competition
       !

       CALL light (npts, dt_days, &
            PFTpresent, cn_ind, lai, maxfpc_lastyear, &
            ind, biomass, veget_lastlight, bm_to_litter)

       !
       ! 11.2 reset attributes for eliminated PFTs
       !

       CALL kill (npts, 'light     ', lm_lastyearmax, &
            ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
            lai, age, leaf_age, leaf_frac, &
            when_growthinit, everywhere, veget, veget_max, bm_to_litter)

    ENDIF

    !
    ! 12 establishment of saplings
    !

    IF ( control%ok_dgvm ) THEN

       !
       ! 12.1 do establishment
       !

       CALL establish (npts, dt_days, PFTpresent, regenerate, &
            neighbours, resolution, need_adjacent, herbivores, &
            precip_lastyear, gdd0_lastyear, lm_lastyearmax, &
            cn_ind, lai, avail_tree, avail_grass, &
            leaf_age, leaf_frac, &
            ind, biomass, age, everywhere, co2_to_bm, veget_max)

       !
       ! 12.2 calculate new crown area (and maximum vegetation cover)
       !

       CALL crown (npts, PFTpresent, &
            ind, biomass, &
            veget_max, cn_ind, height)

    ENDIF

    !
    ! 13 calculate final LAI and vegetation cover.
    !

    CALL cover (npts, cn_ind, ind, biomass, &
         veget_max, veget_max_old, veget, &
         lai, litter, carbon)

    !
    ! 14 the whole litter stuff:
    !    litter update, lignin content, PFT parts, litter decay, 
    !    litter heterotrophic respiration, dead leaf soil cover.
    !    No vertical discretisation in the soil for litter decay.
    !
    ! added by shilong for harvest
    IF(harvest_agri) THEN
       CALL harvest(npts, dt_days, veget_max, veget, &
            bm_to_litter, turnover_daily, &
            harvest_above)
    ENDIF

    ! 15.1  Land cover change

    !shilong adde turnover_daily
    IF(EndOfYear) THEN
       IF (lcchange) THEN
          CALL lcchange_main (npts, dt_days, veget_max, veget_max_new, &
               biomass, ind, age, PFTpresent, senescence, when_growthinit, &
               everywhere, veget, & 
               co2_to_bm, bm_to_litter, turnover_daily, bm_sapl, tree, cn_ind,flux10,flux100, &
!!$               prod10,prod100,prod10_total,prod100_total,&
!!$               convflux,cflux_prod_total,cflux_prod10,cflux_prod100,leaf_frac,&
               prod10,prod100,convflux,cflux_prod10,cflux_prod100,leaf_frac,&
               npp_longterm, lm_lastyearmax, litter, carbon)
       ENDIF
    ENDIF
!MM déplacement pour initialisation correcte des grandeurs cumulées :
    cflux_prod_total(:) = convflux(:) + cflux_prod10(:) + cflux_prod100(:)
    prod10_total(:)=SUM(prod10,dim=2)
    prod100_total(:)=SUM(prod100,dim=2)
    !
    ! 16 total heterotrophic respiration

    tot_soil_carb=zero
    tot_litter_carb=zero
    DO j=2,nvm

       tot_litter_carb(:,j) = tot_litter_carb(:,j) + (litter(:,istructural,j,iabove) + &
            &          litter(:,imetabolic,j,iabove) + &
            &          litter(:,istructural,j,ibelow) + litter(:,imetabolic,j,ibelow))

       tot_soil_carb(:,j) = tot_soil_carb(:,j) + (carbon(:,iactive,j) + &
            &          carbon(:,islow,j)+  carbon(:,ipassive,j))

    ENDDO
    tot_litter_soil_carb = tot_litter_carb + tot_soil_carb

    tot_live_biomass = biomass(:,:,ileaf) + biomass(:,:,isapabove) + biomass(:,:,isapbelow) +&
         &             biomass(:,:,iheartabove) + biomass(:,:,iheartbelow) + &
         &             biomass(:,:,iroot)+ biomass(:,:,ifruit)+ biomass(:,:,icarbres)

    tot_turnover = turnover_daily(:,:,ileaf) + turnover_daily(:,:,isapabove) + &
         &         turnover_daily(:,:,isapbelow) + turnover_daily(:,:,iheartabove) + &
         &         turnover_daily(:,:,iheartbelow) + turnover_daily(:,:,iroot) + &
         &         turnover_daily(:,:,ifruit) + turnover_daily(:,:,icarbres)

    tot_bm_to_litter = bm_to_litter(:,:,ileaf) + bm_to_litter(:,:,isapabove) +&
         &             bm_to_litter(:,:,isapbelow) + bm_to_litter(:,:,iheartbelow) +&
         &             bm_to_litter(:,:,iheartabove) + bm_to_litter(:,:,iroot) + &
         &             bm_to_litter(:,:,ifruit) + bm_to_litter(:,:,icarbres)

    !
    ! 17 history
    !

    ! 2d

    CALL histwrite (hist_id_stomate, 'RESOLUTION_X', itime, &
         resolution(:,1), npts, hori_index)
    CALL histwrite (hist_id_stomate, 'RESOLUTION_Y', itime, &
         resolution(:,2), npts, hori_index)
    CALL histwrite (hist_id_stomate, 'CONTFRAC', itime, &
         contfrac(:), npts, hori_index)

    CALL histwrite (hist_id_stomate, 'LITTER_STR_AB', itime, &
         litter(:,istructural,:,iabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LITTER_MET_AB', itime, &
         litter(:,imetabolic,:,iabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LITTER_STR_BE', itime, &
         litter(:,istructural,:,ibelow), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LITTER_MET_BE', itime, &
         litter(:,imetabolic,:,ibelow), npts*nvm, horipft_index)

    CALL histwrite (hist_id_stomate, 'DEADLEAF_COVER', itime, &
         deadleaf_cover, npts, hori_index)

    CALL histwrite (hist_id_stomate, 'TOTAL_SOIL_CARB', itime, &
         tot_litter_soil_carb, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'CARBON_ACTIVE', itime, &
         carbon(:,iactive,:), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'CARBON_SLOW', itime, &
         carbon(:,islow,:), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'CARBON_PASSIVE', itime, &
         carbon(:,ipassive,:), npts*nvm, horipft_index)

    CALL histwrite (hist_id_stomate, 'T2M_MONTH', itime, &
         t2m_month, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'T2M_WEEK', itime, &
         t2m_week, npts, hori_index)

    CALL histwrite (hist_id_stomate, 'HET_RESP', itime, &
         resp_hetero(:,:), npts*nvm, horipft_index)

    CALL histwrite (hist_id_stomate, 'BLACK_CARBON', itime, &
         black_carbon, npts, hori_index)

    CALL histwrite (hist_id_stomate, 'FIREINDEX', itime, &
         fireindex(:,:), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LITTERHUM', itime, &
         litterhum_daily, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'CO2_FIRE', itime, &
         co2_fire, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'CO2_TAKEN', itime, &
         co2_to_bm, npts*nvm, horipft_index)
    ! land cover change
    CALL histwrite (hist_id_stomate, 'CONVFLUX', itime, &
         convflux, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'CFLUX_PROD10', itime, &
         cflux_prod10, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'CFLUX_PROD100', itime, &
         cflux_prod100, npts, hori_index)
    CALL histwrite (hist_id_stomate, 'HARVEST_ABOVE', itime, &
         harvest_above, npts, hori_index)

    ! 3d

    CALL histwrite (hist_id_stomate, 'LAI', itime, &
         lai, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'VEGET', itime, &
         veget, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'VEGET_MAX', itime, &
         veget_max, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'NPP', itime, &
         npp_daily, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'GPP', itime, &
         gpp_daily, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'IND', itime, &
         ind, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'TOTAL_M', itime, &
         tot_live_biomass, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LEAF_M', itime, &
         biomass(:,:,ileaf), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'SAP_M_AB', itime, &
         biomass(:,:,isapabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'SAP_M_BE', itime, &
         biomass(:,:,isapbelow), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'HEART_M_AB', itime, &
         biomass(:,:,iheartabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'HEART_M_BE', itime, &
         biomass(:,:,iheartbelow), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'ROOT_M', itime, &
         biomass(:,:,iroot), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'FRUIT_M', itime, &
         biomass(:,:,ifruit), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'RESERVE_M', itime, &
         biomass(:,:,icarbres), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'TOTAL_TURN', itime, &
         tot_turnover, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LEAF_TURN', itime, &
         turnover_daily(:,:,ileaf), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'SAP_AB_TURN', itime, &
         turnover_daily(:,:,isapabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'ROOT_TURN', itime, &
         turnover_daily(:,:,iroot), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'FRUIT_TURN', itime, &
         turnover_daily(:,:,ifruit), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'TOTAL_BM_LITTER', itime, &
         tot_bm_to_litter, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'LEAF_BM_LITTER', itime, &
         bm_to_litter(:,:,ileaf), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'SAP_AB_BM_LITTER', itime, &
         bm_to_litter(:,:,isapabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'SAP_BE_BM_LITTER', itime, &
         bm_to_litter(:,:,isapbelow), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'HEART_AB_BM_LITTER', itime, &
         bm_to_litter(:,:,iheartabove), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'HEART_BE_BM_LITTER', itime, &
         bm_to_litter(:,:,iheartbelow), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'ROOT_BM_LITTER', itime, &
         bm_to_litter(:,:,iroot), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'FRUIT_BM_LITTER', itime, &
         bm_to_litter(:,:,ifruit), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'RESERVE_BM_LITTER', itime, &
         bm_to_litter(:,:,icarbres), npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'MAINT_RESP', itime, &
         resp_maint, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'GROWTH_RESP', itime, &
         resp_growth, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'AGE', itime, &
         age, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'HEIGHT', itime, &
         height, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'MOISTRESS', itime, &
         moiavail_week, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'VCMAX', itime, &
         vcmax, npts*nvm, horipft_index)
    CALL histwrite (hist_id_stomate, 'TURNOVER_TIME', itime, &
         turnover_time, npts*nvm, horipft_index)
    ! land cover change
    CALL histwrite (hist_id_stomate, 'PROD10', itime, &
         prod10, npts*11, horip11_index)
    CALL histwrite (hist_id_stomate, 'PROD100', itime, &
         prod100, npts*101, horip101_index)
    CALL histwrite (hist_id_stomate, 'FLUX10', itime, &
         flux10, npts*10, horip10_index)
    CALL histwrite (hist_id_stomate, 'FLUX100', itime, &
         flux100, npts*100, horip100_index)

    IF ( hist_id_stomate_IPCC > 0 ) THEN
       vartmp(:)=SUM(tot_live_biomass*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cVeg", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(tot_litter_carb*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cLitter", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(tot_soil_carb*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cSoil", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=(prod10_total + prod100_total)/1e3
       CALL histwrite (hist_id_stomate_IPCC, "cProduct", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(lai*veget_max,dim=2)*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "lai", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(gpp_daily*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "gpp", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM((resp_maint+resp_growth)*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "ra", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(npp_daily*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "npp", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(resp_hetero*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "rh", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(co2_fire*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "fFire", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=harvest_above/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "fHarvest", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=cflux_prod_total/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "fLuc", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=(SUM((gpp_daily-(resp_maint+resp_growth+resp_hetero)-co2_fire) &
            &        *veget_max,dim=2)-cflux_prod_total-harvest_above)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "nbp", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(tot_bm_to_litter*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "fVegLitter", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(SUM(soilcarbon_input,dim=2)*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "fLitterSoil", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(biomass(:,:,ileaf)*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cLeaf", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM((biomass(:,:,isapabove)+biomass(:,:,iheartabove))*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cWood", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(( biomass(:,:,iroot) + biomass(:,:,isapbelow) + biomass(:,:,iheartbelow) ) &
            &        *veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cRoot", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(( biomass(:,:,icarbres) + biomass(:,:,ifruit))*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cMisc", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM((litter(:,istructural,:,iabove)+litter(:,imetabolic,:,iabove))*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cLitterAbove", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM((litter(:,istructural,:,ibelow)+litter(:,imetabolic,:,ibelow))*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cLitterBelow", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(carbon(:,iactive,:)*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cSoilFast", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(carbon(:,islow,:)*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cSoilMedium", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(carbon(:,ipassive,:)*veget_max,dim=2)/1e3*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "cSoilSlow", itime, &
         vartmp, npts, hori_index)
       DO j=1,nvm
          histvar(:,j)=veget_max(:,j)*contfrac(:)*100
       ENDDO
       CALL histwrite (hist_id_stomate_IPCC, "landCoverFrac", itime, &
         histvar, npts*nvm, horipft_index)
       vartmp(:)=(veget_max(:,3)+veget_max(:,6)+veget_max(:,8)+veget_max(:,9))*contfrac*100
       CALL histwrite (hist_id_stomate_IPCC, "treeFracPrimDec", itime, &
          vartmp, npts, hori_index)
       vartmp(:)=(veget_max(:,2)+veget_max(:,4)+veget_max(:,5)+veget_max(:,7))*contfrac*100
       CALL histwrite (hist_id_stomate_IPCC, "treeFracPrimEver", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=(veget_max(:,10)+veget_max(:,12))*contfrac*100
       CALL histwrite (hist_id_stomate_IPCC, "c3PftFrac", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=(veget_max(:,11)+veget_max(:,13))*contfrac*100
       CALL histwrite (hist_id_stomate_IPCC, "c4PftFrac", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(resp_growth*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "rGrowth", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(resp_maint*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "rMaint", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(bm_alloc(:,:,ileaf)*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "nppLeaf", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(bm_alloc(:,:,isapabove)*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "nppWood", itime, &
         vartmp, npts, hori_index)
       vartmp(:)=SUM(( bm_alloc(:,:,isapbelow) + bm_alloc(:,:,iroot) )*veget_max,dim=2)/1e3/one_day*contfrac
       CALL histwrite (hist_id_stomate_IPCC, "nppRoot", itime, &
         vartmp, npts, hori_index)

       CALL histwrite (hist_id_stomate_IPCC, 'RESOLUTION_X', itime, &
            resolution(:,1), npts, hori_index)
       CALL histwrite (hist_id_stomate_IPCC, 'RESOLUTION_Y', itime, &
            resolution(:,2), npts, hori_index)
       CALL histwrite (hist_id_stomate_IPCC, 'CONTFRAC', itime, &
            contfrac(:), npts, hori_index)

    ENDIF

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving stomate_lpj'

  END SUBROUTINE StomateLpj

  SUBROUTINE harvest(npts, dt_days, veget_max, veget, &
       bm_to_litter, turnover_daily, &
       harvest_above)
    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                            :: npts

    ! Time step (days)
    REAL(r_std), INTENT(in)                                         :: dt_days

    ! new "maximal" coverage fraction of a PFT (LAI -> infinity) on ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                 :: veget_max
    ! 0.2 modified fields

    ! fractional coverage on natural/agricultural ground, taking into
    !   account LAI (=grid-scale fpc)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                    :: veget

    ! conversion of biomass to litter (gC/(m**2 of nat/agri ground)) / day
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)           :: bm_to_litter

    ! Turnover rates (gC/(m**2 of ground)/day)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(inout)          :: turnover_daily
    ! harvest above ground biomass for agriculture
    REAL(r_std), DIMENSION(npts), INTENT(inout)                       :: harvest_above
    ! 0.4 local

    ! indices
    INTEGER(i_std)                                                     :: i, j, k, l, m

    ! biomass increase (gC/(m**2 of ground))
    REAL(r_std)                                                        :: above_old
    ! yearly initialisation
    above_old             = zero
    harvest_above         = zero
    DO i = 1, npts
       DO j=1, nvm
          IF (j > 11) THEN
             above_old = turnover_daily(i,j,ileaf) + turnover_daily(i,j,isapabove) + &
                  &       turnover_daily(i,j,iheartabove) + turnover_daily(i,j,ifruit) + &
                  &       turnover_daily(i,j,icarbres) + turnover_daily(i,j,isapbelow) + &
                  &       turnover_daily(i,j,iheartbelow) + turnover_daily(i,j,iroot)

             turnover_daily(i,j,ileaf) = turnover_daily(i,j,ileaf)*0.55
             turnover_daily(i,j,isapabove) = turnover_daily(i,j,isapabove)*0.55
             turnover_daily(i,j,isapbelow) = turnover_daily(i,j,isapbelow)*0.55
             turnover_daily(i,j,iheartabove) = turnover_daily(i,j,iheartabove)*0.55
             turnover_daily(i,j,iheartbelow) = turnover_daily(i,j,iheartbelow)*0.55
             turnover_daily(i,j,iroot) = turnover_daily(i,j,iroot)*0.55
             turnover_daily(i,j,ifruit) = turnover_daily(i,j,ifruit)*0.55
             turnover_daily(i,j,icarbres) = turnover_daily(i,j,icarbres)*0.55
             harvest_above(i)  = harvest_above(i) + veget_max(i,j) * above_old *0.45

          ENDIF
       ENDDO
    ENDDO

!!$    harvest_above = harvest_above
  END SUBROUTINE harvest
END MODULE stomate_lpj
